@@ -42,14 +42,6 @@ written by
 #endif
 
 
-bool Upload(UriParser& srt, UriParser& file);
-bool Download(UriParser& srt, UriParser& file);
-
-const srt_logging::LogFA SRT_LOGFA_APP = 10;
-
-static size_t g_buffer_size = 1456;
-static bool g_skip_flushing = false;
-
 using namespace std;
 
 static bool interrupt = false;
@@ -59,110 +51,177 @@ void OnINT_ForceExit(int)
     interrupt = true;
 }
 
-int main( int argc, char** argv )
-{
-    set<string>
-        o_loglevel = { "ll", "loglevel" },
-        o_buffer = {"b", "buffer" },
-        o_verbose = {"v", "verbose" },
-        o_noflush = {"s", "skipflush" },
-        o_fullstats = {"f", "fullstats" };
 
-    // Options that expect no arguments (ARG_NONE) need not be mentioned.
-    vector<OptionScheme> optargs = {
-        { o_loglevel, OptionScheme::ARG_ONE },
-        { o_buffer, OptionScheme::ARG_ONE },
-        { o_noflush, OptionScheme::ARG_NONE },
-        { o_fullstats, OptionScheme::ARG_NONE }
+struct FileTransmitConfig
+{
+    unsigned long chunk_size;
+    bool skip_flushing;
+    bool quiet = false;
+    srt_logging::LogLevel::type loglevel = srt_logging::LogLevel::error;
+    set<srt_logging::LogFA> logfas;
+    string logfile;
+    int bw_report = 0;
+    int stats_report = 0;
+    string stats_out;
+    PrintFormat stats_pf = PRINT_FORMAT_2COLS;
+    bool full_stats = false;
+
+    string source;
+    string target;
+};
+
+
+void PrintOptionHelp(const set<string> &opt_names, const string &value, const string &desc)
+{
+    cerr << "\t";
+    int i = 0;
+    for (auto opt : opt_names)
+    {
+        if (i++) cerr << ", ";
+        cerr << "-" << opt;
+    }
+
+    if (!value.empty())
+        cerr << ":" << value;
+    cerr << "\t- " << desc << "\n";
+}
+
+
+int parse_args(FileTransmitConfig &cfg, int argc, char** argv)
+{
+    const set<string>
+        o_chunk     = { "c", "chunk" },
+        o_no_flush  = { "sf", "skipflush" },
+        o_bwreport  = { "r", "bwreport", "report", "bandwidth-report", "bitrate-report" },
+        o_statsrep  = { "s", "stats", "stats-report-frequency" },
+        o_statsout  = { "statsout" },
+        o_statspf   = { "pf", "statspf" },
+        o_statsfull = { "f", "fullstats" },
+        o_loglevel  = { "ll", "loglevel" },
+        o_logfa     = { "logfa" },
+        o_logfile   = { "logfile" },
+        o_quiet     = { "q", "quiet" },
+        o_verbose   = { "v", "verbose" },
+        o_help      = { "h", "help" },
+        o_version   = { "version" };
+
+    const vector<OptionScheme> optargs = {
+        { o_chunk,        OptionScheme::ARG_ONE },
+        { o_no_flush,     OptionScheme::ARG_NONE },
+        { o_bwreport,     OptionScheme::ARG_ONE },
+        { o_statsrep,     OptionScheme::ARG_ONE },
+        { o_statsout,     OptionScheme::ARG_ONE },
+        { o_statspf,      OptionScheme::ARG_ONE },
+        { o_statsfull,    OptionScheme::ARG_NONE },
+        { o_loglevel,     OptionScheme::ARG_ONE },
+        { o_logfa,        OptionScheme::ARG_ONE },
+        { o_logfile,      OptionScheme::ARG_ONE },
+        { o_quiet,        OptionScheme::ARG_NONE },
+        { o_verbose,      OptionScheme::ARG_NONE },
+        { o_help,         OptionScheme::ARG_NONE },
+        { o_version,      OptionScheme::ARG_NONE }
     };
+
     options_t params = ProcessOptions(argv, argc, optargs);
 
-    /*
-    cerr << "OPTIONS (DEBUG)\n";
-    for (auto o: params)
-    {
-        cerr << "[" << o.first << "] ";
-        copy(o.second.begin(), o.second.end(), ostream_iterator<string>(cerr, " "));
-        cerr << endl;
-    }
-    */
+          bool print_help    = Option<OutBool>(params, false, o_help);
+    const bool print_version = Option<OutBool>(params, false, o_version);
 
-    vector<string> args = params[""];
-    if ( args.size() < 2 )
+    if (params[""].size() != 2 && !print_help && !print_version)
     {
-        cerr << "Usage: " << argv[0] << " <source> <target>\n";
+        cerr << "ERROR. Invalid syntax. Specify source and target URIs.\n";
+        if (params[""].size() > 0)
+        {
+            cerr << "The following options are passed without a key: ";
+            copy(params[""].begin(), params[""].end(), ostream_iterator<string>(cerr, ", "));
+            cerr << endl;
+        }
+        print_help = true; // Enable help to print it further
+    }
+
+
+    if (print_help)
+    {
+        cout << "SRT sample application to transmit files.\n";
+        cerr << "SRT Library version: " << SRT_VERSION << endl;
+        cerr << "Usage: srt-file-transmit [options] <input-uri> <output-uri>\n";
+        cerr << "\n";
+
+        PrintOptionHelp(o_chunk, "<chunk=1456>", "max size of data read in one step");
+        PrintOptionHelp(o_no_flush, "", "skip output file flushing");
+        PrintOptionHelp(o_bwreport, "<every_n_packets=0>", "bandwidth report frequency");
+        PrintOptionHelp(o_statsrep, "<every_n_packets=0>", "frequency of status report");
+        PrintOptionHelp(o_statsout, "<filename>", "output stats to file");
+        PrintOptionHelp(o_statspf, "<format=default>", "stats printing format [json|csv|default]");
+        PrintOptionHelp(o_statsfull, "", "full counters in stats-report (prints total statistics)");
+        PrintOptionHelp(o_loglevel, "<level=error>", "log level [fatal,error,info,note,warning]");
+        PrintOptionHelp(o_logfa, "<fas=general,...>", "log functional area [all,general,bstats,control,data,tsbpd,rexmit]");
+        PrintOptionHelp(o_logfile, "<filename="">", "write logs to file");
+        PrintOptionHelp(o_quiet, "", "quiet mode (default off)");
+        PrintOptionHelp(o_verbose, "", "verbose mode (default off)");
+        cerr << "\n";
+        cerr << "\t-h,-help - show this help\n";
+        cerr << "\t-version - print SRT library version\n";
+        cerr << "\n";
+        cerr << "\t<input-uri>  - URI specifying a medium to read from\n";
+        cerr << "\t<output-uri> - URI specifying a medium to write to\n";
+        cerr << "URI syntax: SCHEME://HOST:PORT/PATH?PARAM1=VALUE&PARAM2=VALUE...\n";
+        cerr << "Supported schemes:\n";
+        cerr << "\tsrt: use HOST, PORT, and PARAM for setting socket options\n";
+        cerr << "\tudp: use HOST, PORT and PARAM for some UDP specific settings\n";
+        cerr << "\tfile: file URI or file://con to use stdin or stdout\n";
+
+        return 2;
+    }
+
+    if (Option<OutBool>(params, false, o_version))
+    {
+        cerr << "SRT Library version: " << SRT_VERSION << endl;
+        return 2;
+    }
+
+    cfg.chunk_size    = stoul(Option<OutString>(params, "1456", o_chunk));
+    cfg.skip_flushing = Option<OutBool>(params, false, o_no_flush);
+    cfg.bw_report     = stoi(Option<OutString>(params, "0", o_bwreport));
+    cfg.stats_report  = stoi(Option<OutString>(params, "0", o_statsrep));
+    cfg.stats_out     = Option<OutString>(params, "", o_statsout);
+    const string pf   = Option<OutString>(params, "default", o_statspf);
+    if (pf == "default")
+    {
+        cfg.stats_pf = PRINT_FORMAT_2COLS;
+    }
+    else if (pf == "json")
+    {
+        cfg.stats_pf = PRINT_FORMAT_JSON;
+    }
+    else if (pf == "csv")
+    {
+        cfg.stats_pf = PRINT_FORMAT_CSV;
+    }
+    else
+    {
+        cfg.stats_pf = PRINT_FORMAT_2COLS;
+        cerr << "ERROR: Unsupported print format: " << pf << endl;
         return 1;
     }
 
-    string loglevel = Option<OutString>(params, "error", o_loglevel);
-    srt_logging::LogLevel::type lev = SrtParseLogLevel(loglevel);
-    UDT::setloglevel(lev);
-    UDT::addlogfa(SRT_LOGFA_APP);
+    cfg.full_stats = Option<OutBool>(params, false, o_statsfull);
+    cfg.loglevel   = SrtParseLogLevel(Option<OutString>(params, "error", o_loglevel));
+    cfg.logfas     = SrtParseLogFA(Option<OutString>(params, "", o_logfa));
+    cfg.logfile    = Option<OutString>(params, "", o_logfile);
+    cfg.quiet      = Option<OutBool>(params, false, o_quiet);
 
-   string verbo = Option<OutString>(params, "no", o_verbose);
-   if ( verbo == "" || !false_names.count(verbo) )
-       Verbose::on = true;
-
-    string bs = Option<OutString>(params, "", o_buffer);
-    if ( bs != "" )
-    {
-        ::g_buffer_size = stoi(bs);
-    }
-
-    string sf = Option<OutString>(params, "no", o_noflush);
-    if (sf == "" || !false_names.count(sf))
-        ::g_skip_flushing = true;
-
-    string sfull = Option<OutString>(params, "no", o_fullstats);
-    if (sfull == "" || !false_names.count(sfull))
-        ::transmit_total_stats = true;
-
-    string source = args[0];
-    string target = args[1];
-
-    UriParser us(source), ut(target);
-
-    Verb() << "SOURCE type=" << us.scheme() << ", TARGET type=" << ut.scheme();
-
-    signal(SIGINT, OnINT_ForceExit);
-    signal(SIGTERM, OnINT_ForceExit);
-
-    try
-    {
-        if (us.scheme() == "srt")
-        {
-            if (ut.scheme() != "file")
-            {
-                cerr << "SRT to FILE should be specified\n";
-                return 1;
-            }
-            Download(us, ut);
-        }
-        else if (ut.scheme() == "srt")
-        {
-            if (us.scheme() != "file")
-            {
-                cerr << "FILE to SRT should be specified\n";
-                return 1;
-            }
-            Upload(ut, us);
-        }
-        else
-        {
-            cerr << "SRT URI must be one of given media.\n";
-            return 1;
-        }
-    }
-    catch (std::exception& x)
-    {
-        cerr << "ERROR: " << x.what() << endl;
-        return 1;
-    }
+    if (Option<OutBool>(params, false, o_verbose))
+        Verbose::on = !cfg.quiet;
 
+    cfg.source = params[""].at(0);
+    cfg.target = params[""].at(1);
 
     return 0;
 }
 
+
+
 void ExtractPath(string path, ref_t<string> dir, ref_t<string> fname)
 {
     //string& dir = r_dir;
@@ -197,14 +256,18 @@ void ExtractPath(string path, ref_t<string> dir, ref_t<string> fname)
         // the fixed uri.
         static const size_t s_max_path = 4096; // don't care how proper this is
         char tmppath[s_max_path];
-        char* gwd = getcwd(tmppath, s_max_path);
+#ifdef _WIN32
+        const char* gwd = _getcwd(tmppath, s_max_path);
+#else
+        const char* gwd = getcwd(tmppath, s_max_path);
+#endif
         if ( !gwd )
         {
             // Don't bother with that now. We need something better for
             // that anyway.
             throw std::invalid_argument("Path too long");
         }
-        string wd = gwd;
+        const string wd = gwd;
 
         directory = wd + "/" + directory;
     }
@@ -213,7 +276,8 @@ void ExtractPath(string path, ref_t<string> dir, ref_t<string> fname)
     *fname = filename;
 }
 
-bool DoUpload(UriParser& ut, string path, string filename)
+bool DoUpload(UriParser& ut, string path, string filename,
+              const FileTransmitConfig &cfg, std::ostream &out_stats)
 {
     bool result = false;
     unique_ptr<Target> tar;
@@ -329,12 +393,12 @@ bool DoUpload(UriParser& ut, string path, string filename)
 
         if (connected)
         {
-            vector<char> buf(::g_buffer_size);
-            size_t n = ifile.read(buf.data(), ::g_buffer_size).gcount();
+            vector<char> buf(cfg.chunk_size);
+            size_t n = ifile.read(buf.data(), cfg.chunk_size).gcount();
             size_t shift = 0;
             while (n > 0)
             {
-                int st = srt_send(s, buf.data() + shift, n);
+                int st = tar->Write(buf.data() + shift, n, out_stats);
                 Verb() << "Upload: " << n << " --> " << st
                     << (!shift ? string() : "+" + Sprint(shift));
                 if (st == SRT_ERROR)
@@ -364,7 +428,7 @@ bool DoUpload(UriParser& ut, string path, string filename)
         }
     }
 
-    if ( result && !::g_skip_flushing )
+    if (result && !cfg.skip_flushing)
     {
         assert(s != SRT_INVALID_SOCK);
 
@@ -402,7 +466,8 @@ exit:
     return result;
 }
 
-bool DoDownload(UriParser& us, string directory, string filename)
+bool DoDownload(UriParser& us, string directory, string filename,
+                const FileTransmitConfig &cfg, std::ostream &out_stats)
 {
     bool result = false;
     unique_ptr<Source> src;
@@ -518,17 +583,16 @@ bool DoDownload(UriParser& us, string directory, string filename)
 
         if (connected)
         {
-            vector<char> buf(::g_buffer_size);
-            int n;
+            vector<char> buf(cfg.chunk_size);
 
-            if(!ofile.is_open())
+            if (!ofile.is_open())
             {
                 const char * fn = id.empty() ? filename.c_str() : id.c_str();
                 directory.append("/");
                 directory.append(fn);
                 ofile.open(directory.c_str(), ios::out | ios::trunc | ios::binary);
 
-                if(!ofile.is_open())
+                if (!ofile.is_open())
                 {
                     cerr << "Error opening file [" << directory << "]" << endl;
                     goto exit;
@@ -536,7 +600,7 @@ bool DoDownload(UriParser& us, string directory, string filename)
                 cerr << "Writing output to [" << directory << "]" << endl;
             }
 
-            n = srt_recv(s, buf.data(), ::g_buffer_size);
+            int n = src->Read(cfg.chunk_size, buf, out_stats);
             if (n == SRT_ERROR)
             {
                 cerr << "Download: SRT error: " << srt_getlasterror_str() << endl;
@@ -571,7 +635,8 @@ exit:
     return result;
 }
 
-bool Upload(UriParser& srt_target_uri, UriParser& fileuri)
+bool Upload(UriParser& srt_target_uri, UriParser& fileuri,
+            const FileTransmitConfig &cfg, std::ostream &out_stats)
 {
     if ( fileuri.scheme() != "file" )
     {
@@ -591,10 +656,11 @@ bool Upload(UriParser& srt_target_uri, UriParser& fileuri)
     // Add some extra parameters.
     srt_target_uri["transtype"] = "file";
 
-    return DoUpload(srt_target_uri, path, filename);
+    return DoUpload(srt_target_uri, path, filename, cfg, out_stats);
 }
 
-bool Download(UriParser& srt_source_uri, UriParser& fileuri)
+bool Download(UriParser& srt_source_uri, UriParser& fileuri,
+              const FileTransmitConfig &cfg, std::ostream &out_stats)
 {
     if (fileuri.scheme() != "file" )
     {
@@ -606,6 +672,112 @@ bool Download(UriParser& srt_source_uri, UriParser& fileuri)
     ExtractPath(path, Ref(directory), Ref(filename));
     Verb() << "Extract path '" << path << "': directory=" << directory << " filename=" << filename;
 
-    return DoDownload(srt_source_uri, directory, filename);
+    return DoDownload(srt_source_uri, directory, filename, cfg, out_stats);
 }
 
+
+int main(int argc, char** argv)
+{
+    FileTransmitConfig cfg;
+    const int parse_ret = parse_args(cfg, argc, argv);
+    if (parse_ret != 0)
+        return parse_ret == 1 ? EXIT_FAILURE : 0;
+
+    //
+    // Set global config variables
+    //
+    if (cfg.chunk_size != SRT_LIVE_MAX_PLSIZE)
+        transmit_chunk_size = cfg.chunk_size;
+    stats_writer = SrtStatsWriterFactory(cfg.stats_pf);
+    transmit_bw_report = cfg.bw_report;
+    transmit_stats_report = cfg.stats_report;
+    transmit_total_stats = cfg.full_stats;
+
+    //
+    // Set SRT log levels and functional areas
+    //
+    srt_setloglevel(cfg.loglevel);
+    for (set<srt_logging::LogFA>::iterator i = cfg.logfas.begin(); i != cfg.logfas.end(); ++i)
+        srt_addlogfa(*i);
+
+    //
+    // SRT log handler
+    //
+    std::ofstream logfile_stream; // leave unused if not set
+    if (!cfg.logfile.empty())
+    {
+        logfile_stream.open(cfg.logfile.c_str());
+        if (!logfile_stream)
+        {
+            cerr << "ERROR: Can't open '" << cfg.logfile.c_str() << "' for writing - fallback to cerr\n";
+        }
+        else
+        {
+            UDT::setlogstream(logfile_stream);
+        }
+    }
+
+    //
+    // SRT stats output
+    //
+    std::ofstream logfile_stats; // leave unused if not set
+    if (cfg.stats_out != "" && cfg.stats_out != "stdout")
+    {
+        logfile_stats.open(cfg.stats_out.c_str());
+        if (!logfile_stats)
+        {
+            cerr << "ERROR: Can't open '" << cfg.stats_out << "' for writing stats. Fallback to stdout.\n";
+            return 1;
+        }
+    }
+    else if (cfg.bw_report != 0 || cfg.stats_report != 0)
+    {
+        g_stats_are_printed_to_stdout = true;
+    }
+
+    ostream &out_stats = logfile_stats.is_open() ? logfile_stats : cout;
+
+    // File transmission code
+
+    UriParser us(cfg.source), ut(cfg.target);
+
+    Verb() << "SOURCE type=" << us.scheme() << ", TARGET type=" << ut.scheme();
+
+    signal(SIGINT, OnINT_ForceExit);
+    signal(SIGTERM, OnINT_ForceExit);
+
+    try
+    {
+        if (us.scheme() == "srt")
+        {
+            if (ut.scheme() != "file")
+            {
+                cerr << "SRT to FILE should be specified\n";
+                return 1;
+            }
+            Download(us, ut, cfg, out_stats);
+        }
+        else if (ut.scheme() == "srt")
+        {
+            if (us.scheme() != "file")
+            {
+                cerr << "FILE to SRT should be specified\n";
+                return 1;
+            }
+            Upload(ut, us, cfg, out_stats);
+        }
+        else
+        {
+            cerr << "SRT URI must be one of given media.\n";
+            return 1;
+        }
+    }
+    catch (std::exception& x)
+    {
+        cerr << "ERROR: " << x.what() << endl;
+        return 1;
+    }
+
+
+    return 0;
+}
