@@ -3,6 +3,11 @@
 
 #include <winsock2.h>
 #include <windows.h>
+// HACK: This include is a workaround for a bug in the MinGW headers
+// where pthread.h, which defines _POSIX_THREAD_SAFE_FUNCTIONS,
+// has to be included before time.h so that time.h defines
+// localtime_r correctly
+#include <pthread.h>
 #include <time.h>
 
 #ifdef __cplusplus
