@@ -29,22 +29,31 @@ set internal_options {
 	with-compiler-type=<name> "compiler type: gcc(default), cc, others simply add ++ for C++"
 	with-srt-name=<name> "Override srt library name"
 	with-haicrypt-name=<name> "Override haicrypt library name (if compiled separately)"
-	enable-debug "turn on debug+nonoptimized build mode (if =2, debug+optimized)"
 }
 
 # Options that refer directly to variables used in CMakeLists.txt
 set cmake_options {
     cygwin-use-posix "Should the POSIX API be used for cygwin. Ignored if the system isn't cygwin. (default: OFF)"
+    enable-encryption "Should encryption features be enabled (default: ON)"
+    enable-unittests "Should the unit tests be enabled (default: OFF)"
     enable-c++11 "Should the c++11 parts (srt-live-transmit) be enabled (default: ON)"
-    enable-c-deps "Extra library dependencies in srt.pc for C language (default: OFF)"
+    enable-apps "Should the Support Applications be Built? (default: ON)"
+    enable-testing "Should developer testing applications be built (default: OFF)"
+    enable-c++-deps "Extra library dependencies in srt.pc for C language (default: OFF)"
     enable-heavy-logging "Should heavy debug logging be enabled (default: OFF)"
     enable-logging "Should logging be enabled (default: ON)"
+    enable-debug=<0,1,2> "Enable debug mode (0=disabled, 1=debug, 2=rel-with-debug)"
+    enable-haicrypt-logging "Should logging in haicrypt be enabled (default: OFF)"
+    enable-inet-pton "Set to OFF to prevent usage of inet_pton when building against modern SDKs (default: ON)"
+    enable-code-coverage "Enable code coverage reporting (default: OFF)"
+    enable-monotonic-clock "Enforced clock_gettime with monotonic clock on GC CV /temporary fix for #729/ (default: OFF)"
     enable-profile "Should instrument the code for profiling. Ignored for non-GNU compiler. (default: OFF)"
-    enable-separate-haicrypt "Should haicrypt be built as a separate library file (default: OFF)"
+    enable-relative-libpath "Should applications contain relative library paths, like ../lib (default: OFF)"
     enable-shared "Should libsrt be built as a shared library (default: ON)"
     enable-static "Should libsrt be built as a static library (default: ON)"
-    enable-suflip "Shuld suflip tool be built (default: OFF)"
+    enable-suflip "Should suflip tool be built (default: OFF)"
     enable-getnameinfo "In-logs sockaddr-to-string should do rev-dns (default: OFF)"
+    enable-unittests "Enable unit tests (default: OFF)"
     enable-thread-check "Enable #include <threadcheck.h> that implements THREAD_* macros"
     openssl-crypto-library=<filepath> "Path to a library."
     openssl-include-dir=<path> "Path to a file."
@@ -52,7 +61,11 @@ set cmake_options {
     pkg-config-executable=<filepath> "pkg-config executable"
     pthread-include-dir=<path> "Path to a file."
     pthread-library=<filepath> "Path to a library."
-    use-gnutls "Should use gnutls instead of openssl (default: OFF)"
+    use-busy-waiting "Enable more accurate sending times at a cost of potentially higher CPU load (default: OFF)"
+    use-gnustl "Get c++ library/headers from the gnustl.pc"
+    use-enclib "Encryption library to be used: openssl(default), gnutls, mbedtls"
+    use-gnutls "DEPRECATED. Use USE_ENCLIB=openssl|gnutls|mbedtls instead"
+    use-openssl-pc "Use pkg-config to find OpenSSL libraries (default: ON)"
     use-static-libstdc++ "Should use static rather than shared libstdc++ (default: OFF)"
 }
 
@@ -124,8 +137,15 @@ proc preprocess {} {
 	# Alias to old name --with-gnutls, which enforces using gnutls instead of openssl
 	if { [info exists ::optval(--with-gnutls)] } {
 		unset ::optval(--with-gnutls)
-		set ::optval(--use-gnutls) ON
-		puts "WARNING: --with-gnutls is a deprecated alias to --use-gnutls, please use the latter one"
+		set ::optval(--use-enclib) gnutls
+		puts "WARNING: --with-gnutls is a deprecated alias to --use-enclib=gnutls, please use the latter one"
+	}
+
+	# Alias to old name --use-gnutls, which enforces using gnutls instead of openssl
+	if { [info exists ::optval(--use-gnutls)] } {
+		unset ::optval(--use-gnutls)
+		set ::optval(--use-enclib) gnutls
+		puts "WARNING: --use-gnutls is a deprecated alias to --use-enclib=gnutls, please use the latter one"
 	}
 
 	if { [info exists ::optval(--with-target-path)] } {
