@@ -174,10 +174,10 @@ using namespace srt_logging;
 CPacket::CPacket():
 __pad(),
 m_data_owned(false),
-m_iSeqNo((int32_t&)(m_nHeader[PH_SEQNO])),
-m_iMsgNo((int32_t&)(m_nHeader[PH_MSGNO])),
-m_iTimeStamp((int32_t&)(m_nHeader[PH_TIMESTAMP])),
-m_iID((int32_t&)(m_nHeader[PH_ID])),
+m_iSeqNo((int32_t&)(m_nHeader[SRT_PH_SEQNO])),
+m_iMsgNo((int32_t&)(m_nHeader[SRT_PH_MSGNO])),
+m_iTimeStamp((int32_t&)(m_nHeader[SRT_PH_TIMESTAMP])),
+m_iID((int32_t&)(m_nHeader[SRT_PH_ID])),
 m_pcData((char*&)(m_PacketVector[PV_DATA].dataRef()))
 {
     m_nHeader.clear();
@@ -235,7 +235,7 @@ void CPacket::pack(UDTMessageType pkttype, void* lparam, void* rparam, int size)
    case UMSG_ACK: //0010 - Acknowledgement (ACK)
       // ACK packet seq. no.
       if (NULL != lparam)
-         m_nHeader[PH_MSGNO] = *(int32_t *)lparam;
+         m_nHeader[SRT_PH_MSGNO] = *(int32_t *)lparam;
 
       // data ACK seq. no. 
       // optional: RTT (microsends), RTT variance (microseconds) advertised flow window size (packets), and estimated link capacity (packets per second)
@@ -245,7 +245,7 @@ void CPacket::pack(UDTMessageType pkttype, void* lparam, void* rparam, int size)
 
    case UMSG_ACKACK: //0110 - Acknowledgement of Acknowledgement (ACK-2)
       // ACK packet seq. no.
-      m_nHeader[PH_MSGNO] = *(int32_t *)lparam;
+      m_nHeader[SRT_PH_MSGNO] = *(int32_t *)lparam;
 
       // control info field should be none
       // but "writev" does not allow this
@@ -288,7 +288,7 @@ void CPacket::pack(UDTMessageType pkttype, void* lparam, void* rparam, int size)
 
    case UMSG_DROPREQ: //0111 - Message Drop Request
       // msg id 
-      m_nHeader[PH_MSGNO] = *(int32_t *)lparam;
+      m_nHeader[SRT_PH_MSGNO] = *(int32_t *)lparam;
 
       //first seq no, last seq no
       m_PacketVector[PV_DATA].set(rparam, size);
@@ -297,7 +297,7 @@ void CPacket::pack(UDTMessageType pkttype, void* lparam, void* rparam, int size)
 
    case UMSG_PEERERROR: //1000 - Error Signal from the Peer Side
       // Error type
-      m_nHeader[PH_MSGNO] = *(int32_t *)lparam;
+      m_nHeader[SRT_PH_MSGNO] = *(int32_t *)lparam;
 
       // control info field should be none
       // but "writev" does not allow this
@@ -309,7 +309,7 @@ void CPacket::pack(UDTMessageType pkttype, void* lparam, void* rparam, int size)
       // for extended control packet
       // "lparam" contains the extended type information for bit 16 - 31
       // "rparam" is the control information
-      m_nHeader[PH_SEQNO] |= *(int32_t *)lparam;
+      m_nHeader[SRT_PH_SEQNO] |= *(int32_t *)lparam;
 
       if (NULL != rparam)
       {
@@ -334,12 +334,12 @@ IOVector* CPacket::getPacketVector()
 
 UDTMessageType CPacket::getType() const
 {
-    return UDTMessageType(SEQNO_MSGTYPE::unwrap(m_nHeader[PH_SEQNO]));
+    return UDTMessageType(SEQNO_MSGTYPE::unwrap(m_nHeader[SRT_PH_SEQNO]));
 }
 
 int CPacket::getExtendedType() const
 {
-    return SEQNO_EXTTYPE::unwrap(m_nHeader[PH_SEQNO]);
+    return SEQNO_EXTTYPE::unwrap(m_nHeader[SRT_PH_SEQNO]);
 }
 
 int32_t CPacket::getAckSeqNo() const
@@ -348,7 +348,7 @@ int32_t CPacket::getAckSeqNo() const
    // This field is used only in UMSG_ACK and UMSG_ACKACK,
    // so 'getAckSeqNo' symbolically defines the only use of it
    // in case of CONTROL PACKET.
-   return m_nHeader[PH_MSGNO];
+   return m_nHeader[SRT_PH_MSGNO];
 }
 
 uint16_t CPacket::getControlFlags() const
@@ -357,40 +357,40 @@ uint16_t CPacket::getControlFlags() const
     // which is not used at all in case when the standard
     // type message is interpreted. This can be used to pass
     // additional special flags.
-    return SEQNO_EXTTYPE::unwrap(m_nHeader[PH_SEQNO]);
+    return SEQNO_EXTTYPE::unwrap(m_nHeader[SRT_PH_SEQNO]);
 }
 
 PacketBoundary CPacket::getMsgBoundary() const
 {
-    return PacketBoundary(MSGNO_PACKET_BOUNDARY::unwrap(m_nHeader[PH_MSGNO]));
+    return PacketBoundary(MSGNO_PACKET_BOUNDARY::unwrap(m_nHeader[SRT_PH_MSGNO]));
 }
 
 bool CPacket::getMsgOrderFlag() const
 {
-    return 0!=  MSGNO_PACKET_INORDER::unwrap(m_nHeader[PH_MSGNO]);
+    return 0!=  MSGNO_PACKET_INORDER::unwrap(m_nHeader[SRT_PH_MSGNO]);
 }
 
 int32_t CPacket::getMsgSeq(bool has_rexmit) const
 {
     if ( has_rexmit )
     {
-        return MSGNO_SEQ::unwrap(m_nHeader[PH_MSGNO]);
+        return MSGNO_SEQ::unwrap(m_nHeader[SRT_PH_MSGNO]);
     }
     else
     {
-        return MSGNO_SEQ_OLD::unwrap(m_nHeader[PH_MSGNO]);
+        return MSGNO_SEQ_OLD::unwrap(m_nHeader[SRT_PH_MSGNO]);
     }
 }
 
 bool CPacket::getRexmitFlag() const
 {
     // return false; //
-    return 0 !=  MSGNO_REXMIT::unwrap(m_nHeader[PH_MSGNO]);
+    return 0 !=  MSGNO_REXMIT::unwrap(m_nHeader[SRT_PH_MSGNO]);
 }
 
 EncryptionKeySpec CPacket::getMsgCryptoFlags() const
 {
-    return EncryptionKeySpec(MSGNO_ENCKEYSPEC::unwrap(m_nHeader[PH_MSGNO]));
+    return EncryptionKeySpec(MSGNO_ENCKEYSPEC::unwrap(m_nHeader[SRT_PH_MSGNO]));
 }
 
 // This is required as the encryption/decryption happens in place.
@@ -398,8 +398,8 @@ EncryptionKeySpec CPacket::getMsgCryptoFlags() const
 // crypto flags after encrypting a packet.
 void CPacket::setMsgCryptoFlags(EncryptionKeySpec spec)
 {
-    int32_t clr_msgno = m_nHeader[PH_MSGNO] & ~MSGNO_ENCKEYSPEC::mask;
-    m_nHeader[PH_MSGNO] = clr_msgno | EncryptionKeyBits(spec);
+    int32_t clr_msgno = m_nHeader[SRT_PH_MSGNO] & ~MSGNO_ENCKEYSPEC::mask;
+    m_nHeader[SRT_PH_MSGNO] = clr_msgno | EncryptionKeyBits(spec);
 }
 
 /*
@@ -449,7 +449,7 @@ EncryptionStatus CPacket::decrypt(HaiCrypt_Handle hcrypto)
    m_PacketVector[PV_DATA].iov_len = rc; // In case clr txt size is different from cipher txt
 
    // Decryption succeeded. Update flags.
-   m_nHeader[PH_MSGNO] &= ~MSGNO_ENCKEYSPEC::mask; // sets EK_NOENC to ENCKEYSPEC bits.
+   m_nHeader[SRT_PH_MSGNO] &= ~MSGNO_ENCKEYSPEC::mask; // sets EK_NOENC to ENCKEYSPEC bits.
 
    return ENCS_CLEAR;
 }
@@ -459,7 +459,7 @@ EncryptionStatus CPacket::decrypt(HaiCrypt_Handle hcrypto)
 uint32_t CPacket::getMsgTimeStamp() const
 {
    // SRT_DEBUG_TSBPD_WRAP may enable smaller timestamp for faster wraparoud handling tests
-   return (uint32_t)m_nHeader[PH_TIMESTAMP] & TIMESTAMP_MASK;
+   return (uint32_t)m_nHeader[SRT_PH_TIMESTAMP] & TIMESTAMP_MASK;
 }
 
 CPacket* CPacket::clone() const
@@ -473,23 +473,22 @@ CPacket* CPacket::clone() const
    return pkt;
 }
 
-#if ENABLE_LOGGING
-std::string CPacket::MessageFlagStr()
+// Useful for debugging
+std::string PacketMessageFlagStr(uint32_t msgno_field)
 {
     using namespace std;
 
     stringstream out;
 
-    static const string boundary [] = { "PB_SUBSEQUENT", "PB_LAST", "PB_FIRST", "PB_SOLO" };
-    static const string order [] = { "ORD_RELAXED", "ORD_REQUIRED" };
-    static const string crypto [] = { "EK_NOENC", "EK_EVEN", "EK_ODD", "EK*ERROR" };
-    static const string rexmit [] = { "SN_ORIGINAL", "SN_REXMIT" };
+    static const char* const boundary [] = { "PB_SUBSEQUENT", "PB_LAST", "PB_FIRST", "PB_SOLO" };
+    static const char* const order [] = { "ORD_RELAXED", "ORD_REQUIRED" };
+    static const char* const crypto [] = { "EK_NOENC", "EK_EVEN", "EK_ODD", "EK*ERROR" };
+    static const char* const rexmit [] = { "SN_ORIGINAL", "SN_REXMIT" };
 
-    out << boundary[int(getMsgBoundary())] << " ";
-    out << order[int(getMsgOrderFlag())] << " ";
-    out << crypto[int(getMsgCryptoFlags())] << " ";
-    out << rexmit[int(getRexmitFlag())];
+    out << boundary[MSGNO_PACKET_BOUNDARY::unwrap(msgno_field)] << " ";
+    out << order[MSGNO_PACKET_INORDER::unwrap(msgno_field)] << " ";
+    out << crypto[MSGNO_ENCKEYSPEC::unwrap(msgno_field)] << " ";
+    out << rexmit[MSGNO_REXMIT::unwrap(msgno_field)];
 
     return out.str();
 }
-#endif
